export declare enum HTTPMethod {
    DELETE = "DELETE",
    GET = "GET",
    HEAD = "HEAD",
    OTIONS = "OPTIONS",
    PATCH = "PATCH",
    POST = "POST",
    PUT = "PUT"
}
export declare type BodyOnly = (body: BodyInit) => Promise<void>;
export declare type RouteOnly = (route: string) => Promise<void>;
export declare type RouteAndBodyOnly = (route: string, body: BodyInit) => Promise<void>;
export declare type NoArgs = () => Promise<void>;
declare type FetchData = BodyOnly | RouteOnly | RouteAndBodyOnly | NoArgs;
export declare type FetchCommands = {
    get: RouteOnly & NoArgs;
    post: FetchData;
    patch: FetchData;
    put: FetchData;
    del: FetchData;
    delete: FetchData;
    query: (query: string, variables?: object) => Promise<void>;
    mutate: (mutation: string, variables?: object) => Promise<void>;
    abort: () => void;
};
export declare type DestructuringCommands<TData = any> = [TData | undefined, boolean, any, FetchCommands];
export declare type UseFetchResult<TData = any> = FetchCommands & {
    data?: TData;
    loading: boolean;
    error?: any;
    request: FetchCommands;
};
export declare type UseFetch<TData> = DestructuringCommands<TData> & UseFetchResult<TData>;
export declare type Options = {
    onMount?: boolean;
    timeout?: number;
    url: string;
} & RequestInit;
export declare type OptionsMaybeURL = Omit<Options, 'url'> & {
    url?: string;
};
export declare type OptionsOverwriteWithContext = (options: Options) => Options;
export {};
